/**
 * SimpleBox by Gh0st 
 *
 */

var SimpleBox = function(_options){
	// Zapiszmy ( Użycie na samym dole )
	var simple = this;
	
	if(!_options){
		_options = {}
	}
	
	var options = $.extend({
		url		   : false,
		data       : {},
		
		title      : 'SimpleBox',
		message    : 'This is a simple message',
		fadeout    : 0,
		fadein     : 600,
		modal      : true,
			
		width  : 600,
		height : false,
		
		submitValue  : 'Ok',
		submitAction : false,
		submitClose  : false,
		
		cancelValue        : 'Zamknij',
		cancelAction       : false,
		cancelTime         : 700,
		cancelEsc          : true,
		cancelOnBlockClick : false,
		
		cssBoxClass        : 'simple_box',
		cssTitleClass      : 'simple_title',
		cssBodyClass       : 'simple_body',
		cssContentClass    : 'simple_content',
		cssBottomClass     : 'simple_bottom',
		cssBlockClass      : 'block'
		
	}, _options);
	
	// Utwórzmy poszczególne elementy naszego simplebox'a
	var box     = $(document.createElement('div')).addClass(options.cssBoxClass);
	var title   = $(document.createElement('div')).addClass(options.cssTitleClass);
	var body    = $(document.createElement('div')).addClass(options.cssBodyClass);
	var content = $(document.createElement('div')).addClass(options.cssContentClass);
	var bottom  = $(document.createElement('div')).addClass(options.cssBottomClass);
	
	if(options.modal === true){
		var block   = $(document.createElement('div')).addClass(options.cssBlockClass);
	}
	
	$('.' + options.cssBoxClass).remove();
	
	// Ułużmy w odpowiedniej kolejności...
	box.append(title).append(body.append(content).append(bottom));

	// Konfiguracja Potwierdzania...
	if(options.submitValue !== false){
		var confirm_button = $(document.createElement('button')).addClass('uibutton confirm').html(options.submitValue);
		bottom.append(confirm_button);
	}	
		
	// Konfiguracja zamykania...
	if(options.cancelValue !== false){
		var cancel_button = $(document.createElement('button')).addClass('uibutton close').html(options.cancelValue);
		bottom.append(cancel_button);
	}
	
	// Jeśli nie ma przycisku zamykania ani potwierdzania to można usunąć stopkę
	if(!confirm_button && !cancel_button){
		bottom.hide();
	}
	
	// Sprawdźmy czy zamykanie ma się odbyć ręcznie i czy automatycznie...
	if(options.fadeout > 0){
		setTimeout(function(){
			simple.close();
		}, options.fadeout);
	}
	
	/**
	 * Zmienia nagłówek naszego simplebox
	 *
	 * @param string
	 */
	this.setTitle = function(_title){
		title.html((_title || 'Simple Box'));
	}
	
	/**
	 * Zmienia treść(na białym obszarze) naszego simplebox
	 *
	 * @param string
	 */
	this.setMessage = function(_message){
		content.html(_message || 'This is a simple message');
	}
	
	/**
	 * Wyświetla nasz simplebox
	 *
	 */
	this.show = function(append_to){
		
		append_to = append_to || $('body');
		append_to.append(box);
		
		box.fadeIn(options.fadein);
		
		if(options.modal === true){
			$('body').append(block);
			block.fadeIn(options.fadein);
		}
	}
	
	/**
	 * Zamyka naszego simplebpx'a
	 *
	 *
	 */
	this.close = function(){
		box.fadeOut(options.cancelTime, function(){
			box.remove();
		});
		
		if(options.modal === true){
			block.fadeOut(options.cancelTime, function(){
				block.remove();
			});
		}
	}

	/**
	 * Ustawia odpowiednią szerokość/wysokość + wyśrodkowywuje box...
	 * Dodatkowo sprawdza wykonane akcje...
	 *
	 */
	this.position = function(){
		// Ustawienie wysokości i szerokości
		box.css({
			width  : (options.width  === false ? 'auto' : (options.width  + 'px')),
			height : (options.height === false ? 'auto' : (options.height + 'px'))
		});
	
		// Pozycjonowanie...
		box.css({
			left      : ($(document).outerWidth()-box.outerWidth())/2,
			top       : 60,
			position  : 'fixed'
		});
	}
	
	// Ustawmy wartości domyślne naszego simplebox'a
	if(options.url !== false){
		
		title.hide();
		body.hide();
		
		$.ajax({
			url: options.url,
			dataType: 'JSON',
			data: options.data,
			type: 'POST',
			
			success: function(json){
				simple.setTitle(json.title);
				simple.setMessage(json.message);
				simple.position();	
				
				title.show();
				body.show();
			}
		});
	}
	else{
		this.setTitle(options.title);
		this.setMessage(options.message);
	}
	
	this.position();
	
	// Bindowanie zamykania...
	box.find('.close').on('click', function(){
		if(options.cancelAction !== false && typeof options.cancelAction === 'function'){
			options.cancelAction();
		}
		simple.close();
	});
		
	// Bindowanie potwierdzania...
	bottom.find('.confirm').on('click', function(){
		if(options.submitAction !== false && typeof options.submitAction === 'function'){
			options.submitAction();
		}
		if(options.submitClose === true){
			simple.close();
		}
	});
	
	// Zamykanie podczas wciśniecia klawisza ESC
	if(options.cancelEsc !== false){
		$(document).on('keydown', function(event){
			event = event || window.event;
			if(event.keyCode == 27){
				if(body.css('display') == 'block'){
					if(options.cancelAction !== false && typeof options.cancelAction === 'function'){
						options.cancelAction();
					}
					simple.close();
				}
			}
		});
	}
	
	// Zamykanie poprzez kliknięcie w tło...
	if(options.modal === true && options.cancelOnBlockClick === true){
		block.click(function(){
			simple.close();
		});
	}
}

function notif(message, delay){
	var body = $('body');
	
	var popup = $(document.createElement('div')).addClass('popupWrapper');
	var inner = $(document.createElement('div')).addClass('popupInner').html(message);
	popup.append(inner);
	
	popup.css({
		left : ($(document).outerWidth()-popup.outerWidth())/3,
	});
	
	body.append(popup);
	
	popup.animate({
		opacity: 1.0
	}, 1000, function(){
		setTimeout(function(){
			popup.animate({
				opacity: 0.0
			}, 600, function(){
				popup.remove();
			});
		}, delay);
	});
}