<?php defined('SYSPATH') or die('No direct script access.');

abstract class Member
{
	/**
	 * Załadowni użytkownicy
	 * @static array
	 */
	static $load_members = array();
	
	/**
	 * Dane zalogowanego użytkownika
	 * @static array
	 */
	static $data = array();
	
	/**
	 * Inicjuje skrypt
	 *
	 */
	static public function initialize()
	{	
		Member::$data = Member::load(Member::logged());
		
		if( Member::logged() && !Member::get('ip_addr', FALSE))
		{
			Member::update(Member::logged(), 'ip_addr', Core::request()->getAddr());
		}
	}
	
	/**
	 * Zwraca dane użytkownika jeśli jest zalogowany...
	 *
	 * @param string
	 * 
	 * @access static
	 * @return string|array
	 */
	static public function get($key = NULL, $default = FALSE)
	{		
		if($key === NULL)
		{
			return Member::$data;
		}
		return isset(Member::$data[$key]) ? Member::$data[$key] : $default;
	}
	
	/**
	 * Dodaje nowego użytkownika...
	 *
	 * @param   string
	 *
	 * @access static
	 * @return  array
	 */
	static public function create($data)
	{
	
		// Pobierzmy wszystkie dane
		$login            = $data['username'];
		$password 		  = $data['password'];
		$password_confirm = $data['password_confirm'];
		$email			  = $data['email'];
		$gadugadu		  = $data['gg'];
		
		
		// Sprawdzmy czy zostały wypełnione
		if($login == '' or $password == '' or $password_confirm == '' or $email == '')
		{
			return array(0, 'Musisz uzupełnić wszystkie pola');
		}
			// Sprawdzmy czy login jest poprawny
		if(!Member::checkLogin($login))
		{
			return array(0, 'Login może składać się z 3 do 10 liter lub cyfr, bez polskich znaków i znaktów podobnych do !@#$%^&*()_+=-{}[]|":;.,<>?/');
		}
		// Sprawdzmy czy hasła się zgadzają
		if(Member::hash($password) != Member::hash($password_confirm))
		{
			return array(0, 'Podane hasła nie są identyczne');
		}
		
		// Sprawdzmy czy adres email jest poprawny
		if(!Member::checkEmail($email))
		{
			return array(0, 'Podany adres email nie jest prawidłowy');
		}

		// Czy taki użytkownik już istnieje?
		if(Member::load($login, 'login') != false or Member::load($email, 'email') != false)
		{
			return array(0, 'Taki login lub adres email jest już zajęty');
		}
		
		// Dodajmy wpis do bazy danych
		$query = Core::db()->prepare('INSERT INTO `members_` VALUES(NULL, ?, ?, ?, ?, 0, ?, 0, 0, ?)');
		$query->execute(array($login, Member::hash($password), $email, $gadugadu, Core::setting()->get('default_group_id'), Core::request()->getAddr()));
		
		if($query->rowCount())
		{
			return array(1, 'Konto zostało utworzone pomyślnie');
		}
		return array(0, 'Coś poszło nie tak');
	}
	
	
	/**
	 * Zwraca(id) czy użytkownik jest zalogowany
	 *
	 * @access static
	 * @retun integer
	 */
	static public function logged()
	{
		return Session::get('member_id', FALSE);
	}
	
	/**
	 * Logowanie...
	 *
	 *
	 * @param  string
	 * @param  string
	 *
	 * @access static
	 * @return bool
	 */
	static public function login($username, $password)
	{	
		// aktualny czas
		$time = time();
	
		// Pobierzmy ustawienia
		$login_limit       = Core::setting()->get('login_limit', 5); // limit prób
		$login_time        = Core::setting()->get('login_limit_time', 5) * Date::MINUTE; // czas blokowania w minutach
		$login_notif       = Core::setting()->get('login_limit_notif', 1); // powiadomienie użytkownika
		$login_notif_admin = Core::setting()->get('login_limit_notif_admin', 1); // powiadomienie administratora
		
		// Jeśli czas blokady nie minął i limit logowania jest włączony i przekroczono limit prób logowań
		// Zwróćmy wiadomość do wyświetlenia
		$block = $time - Session::get('login_time', 0);
		
		if(($block < $login_time) && $login_limit > 0 && Session::get('login_count') >= $login_limit)
		{
			return array(0, 'Możliwość logowania zablokowana, odblokowanie nastąpi za ' . ceil( ($login_time - $block) / 60 ) . ' minut');
		}
		// Jeśli czas blokady upłyął i blokada jest włączona
		elseif(($time - Session::get('login_time', 0) >= $login_time) && $login_limit > 0)
		{
			Session::set('login_count', 0);
			Session::set('login_time',  0);
		}
		
		if($username == '' or $password == '')
		{
			return array(0, 'Wypełnij wszystkie pola...');
		}

		$stmt = Core::db()->prepare('SELECT * FROM `members_` where `login`=? and `password`=? LIMIT 1');
		$stmt->execute(array($username, Member::hash($password)));
			
		if(!$stmt->rowCount())
		{
			// Jeśli limit logowań jest włączony
			if($login_limit > 0)
			{
				// Dodajmy próbę zalogowania
				Session::set('login_count', Session::get('login_count', 0) + 1);
				
				// Zaktualizujmy ostatni czas
				Session::set('login_time', $time);
			}
		
			$stmt = Core::db()->prepare('SELECT `login`, `password_set`, `id` FROM `members_` WHERE `login` = ? LIMIT 1');
			$stmt->execute(array($username));
			
			$fetch = $stmt->fetch();
			$count = $stmt->rowCount();
			
			
			// Sprawdźmy czy użytkownik na którego chcemy się aktualnie zalogować istnieje i czy 
			// Przekroczono ilość prób
			if($count && Session::get('login_count') >= $login_limit)
			{
				// Jeśli opcja powiadamiania użytkownika jest włączona
				if($login_notif)
				{
					// Wyślijmy powiadomienie
					MLogs::add(
						'Użytkownik z adresu IP <b>' . Core::request()->getAddr() . '</b> próbował zalogować się na twoje konto ' . 
						Session::get('login_count') . ' razy.', 
						'login', $fetch['id']
					);
				}
				
				// Powiadommy administratora jeśli włączył tą opcje
				if($login_notif_admin)
				{
					Logs::add('Przkroczono maxymalną ilość prób logowań na to konto', $fetch['id'], 'member');
				}
			}
			
			
			// Wyświetlmy informacje...
			if($count && $fetch['password_set'])
			{
				return array(0, 'Podane hasło jest błędne. <br/> Hasło do tego konta zostało zmienione <br/> ' . Date::format($fetch['password_set']) . '( ' . Date::strtime($fetch['password_set']) . ' )');
			}
			
			return array(0, 'Podany login i/lub hasło nie jest poprawny');
		}

		$member_data = $stmt->fetch();
		
		if($member_data['block'])
		{
			return array(0, 'Twoje konto zostało zablokowane');
		}
					
		Session::set('member_id', $member_data['id']);
		Session::set('login_count', 0);
		Session::set('login_time',  0);
		
		return array(1, 'Zostałeś zalogowany. Proszę chwilę poczekać<script>document.location = core.getConfig(\'base_url\');</script>');
	}
	
	/**
	 * Zwraca dane użytkownika lub false jeśli użytkownik nie istnieje...
	 *
	 * @param string|integer
	 * @param string (id|email|login)
	 *
	 * @access static
	 * @return array
	 */
	static public function load($value, $by='id')
	{
		// Wygenerujmy klucz
		$key = $value . $by;
		
		// Jeśli taki użytkownik nie został wcześniej załadowany...
		if(!isset(Member::$load_members[$key]))
		{	
			$query = Core::db()->prepare(
				'SELECT m.* , g.`group_access` , g.`group_style` , g.`group_name` FROM `members_` m, `groups_` g WHERE m.`'. $by .'` = ? AND g.`id` = m.`group_id` LIMIT 1'
			);
			
			$fetch = false;
			
			$query->execute(array($value)); 
			if($query->rowCount()) // Jeśli znaleziono
			{
				// Pobierzmy dane
				$fetch = $query->fetch();
					
				// Prawa dostępu.
				$fetch['group_access'] =  unserialize($fetch['group_access']);
				
				// Wskazujący
				$member_style = 'cursor: pointer;';
				
				// Nazwa do wyświetlenie, użytkownika.
				$fetch['display_name'] = "<span style='{$member_style} {$fetch['group_style']}'>{$fetch['login']}</span>";
			}
			Member::$load_members[$key] = $fetch;
		}
		// Zwróćmy dane użytkownika...
		return Member::$load_members[$key];
	}
	
	/**
	 * Validating username(login)
	 *
	 * 
	 * @param string
	 *
	 * @access static
	 * @return bool
	 */
	static public function checkLogin($login)
	{
		return preg_match('/^[a-zA-Z0-9]{3,10}$/', $login);
	}
	
	/**
	 * Validating email adress
	 *
	 * @param string
	 *
	 * @access static
	 * @return bool
	 */
	static public function checkEmail($email)
	{
		return filter_var($email, FILTER_VALIDATE_EMAIL);
	}
	
	/**
	 * Haszuje hasła
	 *
	 *
	 * @param string
	 * 
	 * @access static
	 * @return string
	 */
	static public function hash($string)
	{
		return sha1($string);
	}
	
	/**
	 * Update...
	 *
	 * @param integer
	 * @param string
     * @param string	 
	 *
	 * @access static
	 * @return bool
	 */
	static public function update($member_id, $key, $value)
	{	
		return Core::db()->prepare('UPDATE `members_` SET `'.$key.'`=? where `id`=?')
				->execute(array($value, $member_id));
	}
	
	
	/**
	 * Zwraca tablice z wszystkimi użytkownikami według loginu czy też np adresu email...
	 * 
	 *
	 * @param string
	 *
	 * @access static
	 * @return array
	 */
	static public function getAll($start, $offset = 10, $search = '')
	{
		$search = Member::searchString($search);
		
		return Core::db()->query("SELECT * FROM `members_` {$search} LIMIT {$start}, {$offset}")->fetchAll();
	}
	
	/**
	 * Generuje string z dyrektywą LIKE dla mysql :)
	 *
	 * @param string
	 *
	 *
	 * @access static
	 * @return string
	 */
	static public function searchString($search)
	{
		if(!strlen($search))
		{
			return FALSE;
		}
	
		$search = addslashes( $search );
		
		// return...
		return "WHERE `login` LIKE '%{$search}%' or `email` LIKE '%{$search}%' or `gg` LIKE '%{$search}%'";
	}
	
	/**
	 * Usuwa użytkownika...
	 *
	 * @param integer
	 *
	 * @access static
	 * @return integer
	 */
	static public function delete($member_id)
	{
		// Sprawdźmy czy zalogowany użytkownik to nie czasami użytkownik do usunięcia...
		if(Member::logged() == $member_id)
		{
			return array(0, 'Nie możesz usunąć swojego konta.');
		}
		
		$stmt = Core::db()->prepare('DELETE FROM `members_` where `id` = ?');
		
		if($stmt->execute(array($member_id)))
		{
			if($stmt->rowCount())
			{
				return array(1, 'Usunięto użytkownika');
			}
			else
			{
				return array(0, 'Użytkownik nie istnieje');
			}
		}
		return array(0, 'Coś poszło nie tak');
	}
	
	/**
	 * Blokuje użytkownika
	 *
	 * @param integer
	 *
	 * @access static
	 * @return bool
	 */
	static public function block($member_id)
	{
		if(Member::logged() == $member_id)
		{
			return array(0, 'Nie możesz sam siebie zablokować...');
		}
		
		$status = Member::load($member_id);
		if(!$status)
		{
			return array(0, 'Taki użytkownik nie istnieje...');
		}
		
		$status = $status['block'] ? 0 : 1;
		
		if(Member::update($member_id, 'block', $status))
		{
			return array(1, 'Użytkownik jest teraz: ' . ($status ? 'Zablokowany' : 'Odblokowany'), $status);
		}
		
		return array(0, 'Coś poszło nie tak');
	}
	
	/**
	 * Zwraca ilość wszystkich użytkowników w bazie danych...
	 *
	 * @access static
	 * @return integer
	 */
	static public function getMembersCount( $search = '' )
	{
		$q = Core::db()->query('SELECT COUNT(`id`) AS `count` FROM `members_` ' . Member::searchString($search))->fetch();
		
		return $q['count'];
	}
	
	/**
	 * Zwraca łączny stan portfela wszystkich użytkowników
	 *
	 * @access static
	 * @return float
	 */
	static public function getWalletSum()
	{
		$q = Core::db()->query('SELECT SUM( `wallet` ) AS `total` FROM `members_`')->fetch();
		return $q['total'];
	}
	
	/**
	 * Zwraca stan portfela $count użytkowników
	 *
	 * @param integer
	 *
	 * @access static
	 * @return float
	 */
	static public function getWalletTop($count = 3)
	{
		return Core::db()->query('SELECT `login`, `wallet` FROM `members_` order by `wallet` desc LIMIT ' . intval($count))->fetchAll();
	}
	
	/**
	 * Zwraca ilość zablokowanych użytkowników
	 *
	 * @access static
	 * @return integer
	 */
	static public function getBlocksCount()
	{
		$q = Core::db()->query('SELECT COUNT(`block`) AS `count` FROM `members_` where `block` = 1');
		$q = $q->fetch();
		
		return $q['count'];
	}
	
	/**
	 * Nazwa do wyświetlenia
	 *
	 * @param string|integer
	 * @return string
	 */
	static public function getDisplayName($member, $by = 'id')
	{
		// Załadujmy użytkownika
		$member = Member::load($member, $by);
		
		// Sprawdzmy czy użytkownik istnieje
		if(!isset($member['display_name']))
		{
			// Jeśli nie
			return $member . ' nie istnieje';
		}
		
		// Jeśli tak
		return $member['display_name'];
	}
	
	/**
	 * Zwraca czy użytkownik zalogowany posiada takie prawo dostępu
	 *
	 * @param string
	 * @return bool
	 */
	static public function hasAccess($access_name)
	{
		$data = Member::get('group_access');
		
		return $data[$access_name];
	}
	
	// ---------------------------------------------------------------------------------------------------------------------------------------------------- //
	// ----------------------------------------------------------- GRUPY ---------------------------------------------------------------------------------- //
	// ---------------------------------------------------------------------------------------------------------------------------------------------------- //
	
	/**
	 * Lista grup...
	 *
	 * @access static
	 * @return array
	 */
	static public function groupList()
	{
		return Core::db()->query('SELECT * FROM `groups_`')->fetchAll();
	}
	
	/**
	 * Lista uprawieniń
	 *
	 * @access static
	 * @return array
	 */
	static public function getAccessList()
	{
		return array(
			'acp', 'acp_setting', 'acp_amxbans', 'acp_payments', 'acp_members', 'acp_groups', 'acp_logs', 'acp_code', 'acp_board',
			'acp_service', 'news_add', 'news_delete', 'news_edit', 'acp_board'
		);
	}
	
	/**
	 * Dodaje grupe
	 *
	 * @param string
	 * @param string
	 * @param array
	 *
	 * @access static
	 * @return array
	 */
	static public function addGroup($name, $css, $access)
	{
		if($name == '' or !is_array($access))
		{
			return array(0, 'Musisz podać nazwę');
		}
		
		if(Member::issetGroup($name))
		{
			return array(0, 'Podobna grupa już istnieje w bazie danych');
		}
		
		$serialize = serialize($access);
		
		$stmt = Core::db()->prepare('INSERT INTO `groups_` VALUES(NULL, ?, ?, ?)');
		if($stmt->execute(array($name, $serialize, $css)))
		{
			return array(1, 'Grupa została dodana <script>menu(\'groups\', $(\'.groups\'))</script>');
		}
		return array(0, 'Błąd MYSQL');
	}
	
	/**
	 * Zwraca konkretną grupe...
	 *
	 * @param integer
	 * 
	 * @access static
	 * @return array
	 */
	static public function getGroup($id)
	{
		$stmt = Core::db()->prepare('SELECT * FROM `groups_` where `id`= ?');
		$stmt -> execute(array($id));
		
		return $stmt->fetch();
	}
	
	/**
	 * Edytuje grupe...
	 *
	 * @param string
	 * @param string
	 * @param array
	 *
	 * @access static
	 * @return bool
	 */
	static public function editGroup($name, $style, $access, $id)
	{
		if($name == '' or !is_array($access))
		{
			return array(0, 'Wypełnij wszystkie pola');
		}
		
		$access = serialize($access);
		
		$stmt = Core::db()->prepare('UPDATE `groups_` SET `group_name` = ?, `group_style` = ?, `group_access` = ? where `id` = ?');
		$stmt->execute(array($name, $style, $access, $id));
		
		if($stmt->rowCount())
		{
			return array(1, 'Grupa została zmodyfikowana <script>menu(\'groups\', $(\'.groups\'))</script>');
		}
		
		return array(0, 'Nic nie zostało zmodyfikowanego');
	}
	
	/**
	 * Usuwa grupe
	 *
	 * @param integer
	 *
	 * @access static
	 * @return integer
	 */
	static public function deleteGroup($id)
	{
		if(Member::get('group_id') == $id)
		{
			return array(0, 'Nie możesz usunąć swojej grupy');
		}
	
		// Jeśli jakiś użytkownik jest członkiem tej grupy to zmieńmy jego grupe na domyślną...
		// aby uniknąć błędów
		$query = Core::db()->prepare('UPDATE `members_` SET `group_id` = ? where `group_id` = ?');
		$query -> execute(array(Core::get('Settings')->get('default_group_id'), $id));
	
	
		$query = Core::db()->prepare('DELETE FROM `groups_` where `id` = ?');
		$query->execute(array($id));
		return array($query->rowCount(), $query->rowCount() ? 'Usunięto grupe' : 'Taka grupa już nie istnieje');
	}
	
	/**
	 * Zwraca czy dana grupa istnieje czy też nie
	 *
	 * @param string
	 *
	 * @access static
	 * @return bool
	 */
	static public function issetGroup($name)
	{
		// Pobierzmy wszystkie grupy...
		$list = Member::groupList();
		
		foreach($list as $g)
		{
			if(strtolower($name) == strtolower($g['group_name']))
			{	// Jeśli taka grupa istnieje to zwróćmy true
				return true;
			}
		}
		return false;
	}
}