<?php defined('SYSPATH') or die('No direct script access.');

abstract class Menu
{	
	/**
	 * Zwraca liste/wybrane menu
	 *
	 * @param integer
	 * 
	 * @access static
	 * @return array
	 */
	static public function get($id = 0)
	{
		if($id > 0)
		{
			$stmt = Core::db()->prepare('SELECT * FROM `menu_` where `id` = ?');
			$stmt->execute(array($id));
			return $stmt->fetch();
		}
		return Core::db()->query('SELECT * FROM `menu_`')->fetchAll();
	}
	
	/**
	 * Dodaje nową pozycje do menu
	 * 
	 * @param array
	 *
	 * @access static
	 * @return array
	 */
	static public function add(array $data)
	{
		$data = Menu::check($data);
		if(!$data)
		{
			return array(0, 'Nie podałeś adresu/nazwy menu');
		}
		
		$stmt = Core::db()->prepare('INSERT INTO `menu_` VALUES(NULL, ?, ?, ?, ?, ?, ?)');
		$stmt->execute(array($data['name'], $data['url'], $data['float'], $data['admin'], $data['member'], $data['guest']));
		
		return $stmt->rowCount() ? array(1, 'Dodano nową pozycje do menu') : array(0, 'Nie udało się dodać nowej pozycji do menu');
	}
	
	
	/**
	 * Edytuje pozycje z menu
	 * 
	 * @param array
	 *
	 * @access static
	 * @return array
	 */
	static public function edit(array $data)
	{
		$data = Menu::check($data);
		if(!$data)
		{
			return array(0, 'Nie podałeś adresu/nazwy menu');
		}
		
		$stmt = Core::db()->prepare('UPDATE `menu_` SET `name` = ?, `url` = ?, `float` = ?, `admin` = ?, `member` = ?, `guest` = ? WHERE `id` = ?');
		
		$stmt->execute(array($data['name'], $data['url'], $data['float'], $data['admin'], $data['member'], $data['guest'], $data['id']));
		
		return $stmt->rowCount() ? array(1, 'Zmodyfikowano tą pozycje menu') : array(0, 'Żadne dane nie zostały zmienione');
	}
	
	/**
	 * Sprawdza czy wszystkie opcje zostały podane...
	 *
	 * @param array
	 * 
	 * @access static
	 * @return array|false
	 */
	static public function check(array $data)
	{
		$field = array('name', 'url');
		foreach($field as $f)
		{
			if(empty($data[$f]))
			{
				return false;
			}
		}
		
		$data['admin']  = isset($data['admin'])      ? 1 : 0;
		$data['member'] = isset($data['member'])     ? 1 : 0;
		$data['guest']  = isset($data['guest'])      ? 1 : 0;
		
		$data['float']  = isset($data['float']) && $data['float'] == 1 ? 'left' : 'right';
		
		return $data;
	}
	
	/**
	 * Usuwa pozycje z menu
	 *
	 * @param integer
	 *
	 * @access static
	 * @return array
	 */
	static public function delete($id)
	{
		$stmt = Core::db()->prepare('DELETE FROM `menu_` WHERE `id` = ?');
		$stmt->execute(array($id));
		
		if($stmt->rowCount())
		{
			return array(1, 'Usunięto pozycje z menu');
		}
		return array(0, 'Taka pozycja nie istnieje');
	}
}